﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.Pipes;

/**************************NOTES*****************************
 * - Each function will need to write to the file that the
 * Recursive Maze will read.
 * - The pipe name is rmgPipe
 * - MUST CALL StartConnection() BEFORE any other functions
 * 
*************************************************************/
namespace MazeFunctionLibrary
{
    
    public interface rmgLibrary
    {
        //int Add(int Number1, int Number2);
        //Will need basic functions for movement
        //AND for setting up the pipes
        void StartConnection();
        //void StartReadConnection();
        void EndConnections();
        void MoveSetLocation(int row, int column);
        void MoveUp();
        void MoveDown();
        void MoveRight();
        void MoveLeft();
        int CheckWall(int direction);
        int CheckPathUp();
        int CheckPathDown();
        int CheckPathLeft();
        int CheckPathRight();
        int CheckWallUp();
        int CheckWallDown();
        int CheckWallLeft();
        int CheckWallRight();
        int CheckEnd();
        int GetCurRow();
        int GetCurCol();

        
    };
    //Interface Implementation
    public class ManagedClass : rmgLibrary
    {
        //Set up pipe to send data to the client
        const string PIPESENDING = "rmgPipeServer";
        System.IO.Pipes.NamedPipeServerStream pipeStreamWrite = new NamedPipeServerStream(PIPESENDING);
        //StreamWriter sw = new StreamWriter(pipeStreamWrite);
        
        //Set up pipe to receive data from the client
        const string PIPEREADING = "rmgPipeClient";
        System.IO.Pipes.NamedPipeClientStream pipeStreamRead = new NamedPipeClientStream(PIPEREADING);
        

        //public int Add(int Number1, int Number2)
        //{
        //    return Number1 + Number2;
        //}

        //Connection Fucntions

        //*********************************StartWriteConnection****************************
        // This function will be used to create the Connections to the pipes. It will
        // first create the writing pipe and then wait for connection before it connects
        // to the reading pipe. This function is required to create the connecion between
        // the maze and the server.
        //*********************************************************************************
        public void StartConnection()
        {
            //Create the named pipe to send data
           // using (pipeStreamWrite)
           // {
                //PUT TRY CATCH HERE
                Console.WriteLine("[Server] Waiting for client connection");
                pipeStreamWrite.WaitForConnection();
                

                //pipe has been connected
                Console.WriteLine("[Server] Write Connection Established");
           // }

            pipeStreamRead.Connect(120000); //Wait for up to two minutes to connect.
            if (pipeStreamRead.IsConnected == false)
            {
                Console.WriteLine("[Server] Unable to connect Read Pipe to the client");
            }
            else
            {
                Console.WriteLine("[Server] Read Connection Established.");
            }
           
        }

        //public void StartReadConnection()
        //{
        //    pipeStreamRead.Connect();
        //}

        //***********************************EndConnection*********************************
        // This function will close both of the pipes that are connecting to the maze
        // generator from the server side. This should be called before the server program
        // ends in order to release the resources that were used.
        //*********************************************************************************
        public void EndConnections()
        {
            //Close the pipe connection on the Server Side
            if (pipeStreamRead.IsConnected)
            {
                pipeStreamRead.Close();
            }
            if (pipeStreamWrite.IsConnected)
            {
                pipeStreamWrite.WaitForPipeDrain();
                pipeStreamWrite.Close();
            }
        }

        //Movement Functions
        //Each of these functions will send a code through the
        //pipe in order to inform the maze what direction to
        //move or where to place the cursor.

        //**********************************MoveSetLocation*******************************
        // This function will place the cursor in a set place in the maze. For this it
        // will require two parameters both of which are integers. The first will be used
        // for the row and the second will be for the column. The function will then send
        // the code for the move to the client and then will send the row followed by the
        // column. Nothing will be returned.
        //********************************************************************************
        public void MoveSetLocation(int row, int column)
        {
            string temp = "moveLoc" + "," + row.ToString() + "," + column.ToString();
            SendMessage(temp);
            //string loc = row.ToString() + "," + column.ToString();
            //SendMessage(loc);
            
            //SendMessage(row.ToString());
            //SendMessage(column.ToString());
        }

        //************************************MoveUp*******************************
        // This function will send the client the code for Moving up one place in
        // the maze. Nothing will be returned.
        //*************************************************************************
        public void MoveUp()
        {
            //Send the move up code (1)
            string temp = "up";
            SendMessage(temp);

        }

        //************************************MoveDown*******************************
        // This function will send the client the code for Moving down one place in
        // the maze. Nothing will be returned.
        //***************************************************************************
        public void MoveDown()
        {
            string temp = "down";
            SendMessage(temp);
        }

        //************************************MoveRight*******************************
        // This function will send the client the code for Moving right one place in
        // the maze. Nothing will be returned.
        //****************************************************************************
        public void MoveRight()
        {
            string temp = "right";
            SendMessage(temp);
        }

        //************************************MoveLeft*******************************
        // This function will send the client the code for Moving left one place in
        // the maze. Nothing will be returned.
        //***************************************************************************
        public void MoveLeft()
        {
            string temp = "left";
            SendMessage(temp);
        }

        //***********************CheckWall**************************
        // This function will return an int based on whether there
        // is a wall in the specified direction within one step.
        // The function will return a -1 if there is an error or
        // the movement was out of the maze, a 0 if it was a wall,
        // or a 1 if it was a path.
        // For example if the path is a single line then this
        // function will return a 0 for the walls on the sides
        // of the cursor while the open paths in front and behind
        // will return a 1.
        //*********************************************************
        public int CheckWall(int direction)
        {
            string temp = "wallCheck";
            SendMessage(temp);
            SendMessage(direction.ToString()); //Must send which direction to check
            
            //Obtain the rersult
            return Convert.ToInt32(ReadMessage());
        }

        //*********************************CheckWallUp***************************
        // This function will check and see if there is a wall one unit
        // Up from the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallUp()
        {
            string temp = "checkWallUp";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //*********************************CheckWallDown************************
        // This function will check and see if there is a wall one unit
        // Down from the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallDown()
        {
            string temp = "checkWallDown";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //*********************************CheckWallLeft************************
        // This function will check and see if there is a wall one unit
        // Left from the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallLeft()
        {
            string temp = "checkWallLeft";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //*********************************CheckWallRight************************
        // This function will check and see if there is a wall one unit
        // Right from the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallRight()
        {
            string temp = "checkWallRight";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //*****************************GetCurRow*************************
        // This function will send the code for retrieving the current
        // row to the client which will then return the row number of the
        // current position in the maze.
        //***************************************************************
        public int GetCurRow()
        {
            string temp = "curRow";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //***************************GetCurCol**********************
        // This function will send the code for retrieving the
        // current column to the client which will then return the
        // column number of the current position in the maze.
        //**********************************************************
        public int GetCurCol()
        {
            string temp = "curCol";
            SendMessage(temp);
            
            return Convert.ToInt32(ReadMessage());
        }

        //*****************************CheckEnd**********************
        // This function will check and see if the end of he maze
        // has been reached or not. It will return a 1 if it has
        // or a 0 if it has not.
        //***********************************************************
        public int CheckEnd()
        {
            string temp = "endCheck";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //**********************************CheckPathUp***************************
        // This function will check and see what the value of the maze square
        // one unit up from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //************************************************************************
        public int CheckPathUp()
        {
            string temp = "checkUp";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //**********************************CheckPathDown*************************
        // This function will check and see what the value of the maze square
        // one unit down from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //************************************************************************
        public int CheckPathDown()
        {
            string temp = "checkDown";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //**********************************CheckPathLeft************************
        // This function will check and see what the value of the maze square
        // one unit Left from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //************************************************************************
        public int CheckPathLeft()
        {
            string temp = "checkLeft";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //**********************************CheckPathRight***************************
        // This function will check and see what the value of the maze square
        // one unit Right from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //************************************************************************
        public int CheckPathRight()
        {
            string temp = "checkRight";
            SendMessage(temp);
            return Convert.ToInt32(ReadMessage());
        }

        //**********************************SendMessage***************************
        // This function is used to send messages to the client program. However
        // it will only be called by the functions and will not be called by the
        // user.
        //************************************************************************
        private void SendMessage(string message)
        {

            StreamWriter sw = new StreamWriter(pipeStreamWrite);
            
                sw.AutoFlush = true;
                sw.WriteLine(message);
            
        }//End of SendMessage

        //*******************************ReadMessage******************************
        // This function will retrieve messages from the client. However it will
        // not be called by the user and will only be used by the functions
        // themselves.
        //************************************************************************
        private string ReadMessage()
        {

            StreamReader sr = new StreamReader(pipeStreamRead);
            
                return sr.ReadLine();
            
        }//End ReadMessage

    }
}
