﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using Microsoft.Win32;


namespace RecursiveMazeMain
{
    class MazeDoc
    {
        //Basic Document Data
        //Obtained from the MyDocumentClass in lab 6 the DocView lab
        private bool modified = false;
        private string fileName = "";
        private string defaultFileName = "Untitled";
        private string fileExtension = "rmg";
        private string fileExtensionDesc = "Recursive Maze Generator";
        private bool registerFileExtension = true;
        private string filter = "RMG Files (*.rmg) |*.rmg";


        //Specific Data
        private int rows;
        private int columns;
        private bool mazeCreated;
        private int[,] mazeBoard; //The maze setup data
        private int curXLoc;
        private int curYLoc;
        private int finishXLoc;
        private int finishYLoc;
        private string moveData;    //Used to send back the movement to the move list box

        //Specific Constants
        //private const int MAXROWS = 10;
        //private const int MAXCOLUMNS = 10;
        private const int WALLCOLOR = 0;    //Gray
        private const int PASSABLECOLOR = 1;    //Light Gray to White
        private const int BEENPATHCOLOR = 2;    //Green
        private const int ERRORCOLOR = 3;   //Red
        private const int CURRENTLOCCOLOR = 4;  //Gold
        private const int FINISHCOLOR = 6;  //Royal Blue
        private const int FINISHCURRENTCOLOR = 8;   //Blue
        private const int ERRORCURRENTLOCCOLOR = 7;  //Orange

        //Will have server part read from a specified path
        private const string TEMPFILENAMEFROMMAZE = "tmpMazeToServerCom.txt";
        private const string TEMPFILENAMEFROMSERVER = "tempServerToMazeCom.txt";


        //Events
        public event EventHandler UpdateMoveLog;

        //Properties
        //Get and Set functions for the various data

        public int Rows
        {
            get
            {
                return rows;
            }
        }

        public int Columns
        {
            get
            {
                return columns;
            }
        }

        public bool MazeCreated
        {
            get
            {
                return mazeCreated;
            }
        }

        public int[,] MazeBoard
        {
            get
            {
                return mazeBoard;
            }
        }

        public int CurXLoc
        {
            get
            {
                return curXLoc;
            }
            set
            {
                curXLoc = value;
            }
        }

        public int CurYLoc
        {
            get
            {
                return curYLoc;
            }
            set
            {
                curYLoc = value;
            }
        }

        public string MoveData
        {
            get
            {
                return moveData;
            }
        }


        //Events
        public event EventHandler OpenDocument;


        public MazeDoc()
        {
            modified = false;
            mazeCreated = false;    //To tell if maze has been created or loaded

        }


        //******************************Open********************************
        // This funtion will be used by the open command in the main form
        // in order to call the open work function which will display the
        // open dialog which will be used to open existing mazes.
        //******************************************************************
        public bool Open()
        {
            return Open(null);
        }

        //******************************OpenWork*************************
        // This function will do all of the work to display the open
        // dialog. The Basic outline for this function is from the
        // CSCI 350 Lab 6 DocVeiw by Dr. Bonnie Mcvey.
        //***************************************************************
        private bool Open(string newFileName)
        {
            //open the open screen
            OpenFileDialog dlg = new System.Windows.Forms.OpenFileDialog();
            if (string.IsNullOrEmpty(newFileName))
            {
                // Set up dialog
                dlg.DefaultExt = fileExtension;
                dlg.Filter = filter;

                // Get the file to open
                DialogResult res = dlg.ShowDialog();
                if (res != DialogResult.OK)
                {
                    return false;
                }
                newFileName = dlg.FileName;
            }

            try
            {
                //Obtain the data from the file
                using (StreamReader sr = new StreamReader(newFileName))
                {
                                       
                    //read the number of rows and columns
                    string rowCol = sr.ReadLine();
                    rows = Convert.ToInt32(rowCol);
                    rowCol = sr.ReadLine();
                    columns = Convert.ToInt32(rowCol);

                    //Retrieve the Maze Data
                    //So the walls, paths, start, and finish

                    //First Create a new instance of the maze board
                    //with the obtained rows and columns data
                    mazeBoard = new int[rows, columns];

                    string mazeData;    //Data read from file

                    //obtain the values for each item in the maze
                    for (int i = 0; i < rows; i++)    //The number of times to get all rows
                    {
                        mazeData = sr.ReadLine();   //Get next line of data
                        string[] mazeDataSplit = mazeData.Split(' ');
                        for (int j = 0; j < columns; j++) //Number needed to get all columns
                        {
                            mazeBoard[i, j] = Convert.ToInt32(mazeDataSplit[j]);
                            if(Convert.ToInt32(mazeDataSplit[j]) == 5)
                            {
                                curXLoc = i;
                                curYLoc = j;
                            }
                            if (Convert.ToInt32(mazeDataSplit[j]) == 6)
                            {
                                finishXLoc = i;
                                finishYLoc = j;
                            }
                        }
                    }
                    

                        
                 }
                    
            }
            catch (Exception e)
            {
                MessageBox.Show("Can't open file: " + newFileName + "\r\n" + e.Message, Application.ProductName);
                return false;
            }
        
            //More will follow   
            // Success
            if (this.OpenDocument != null)
                this.OpenDocument(this, EventArgs.Empty);
            mazeCreated = true;
            return true;
        
        }//End of OpenWork


        //*******************************Save****************************
        // This function will call the SaveWork function. It will
        // recieve a 2d int array which will hold the information that
        // hold the maze data and an int for a row and a column. It will
        // then send the information to the work function.
        //***************************************************************
        public bool Save(int[,] tempMaze, int row, int col)
        {
            return SaveWork(tempMaze, rows, col);
        }//End of Save

        //*******************************SaveWork*************************
        // This function will save the new maze to a file. It will recieve
        // the maze data via a 2d array and the row and column data via
        // integers. It will then open the save dialog which will allow
        // the user to decide where to save the maze and what it will be
        // called. The function will fist save the row and column to the
        // maze and will then save the data for what is in the maze.
        //****************************************************************
        private bool SaveWork(int[,] tempMaze, int row, int col)
        {
            string newFileName = fileName;
            SaveFileDialog sdlg = new SaveFileDialog();
                sdlg.DefaultExt = fileExtension;
                sdlg.Filter = filter;
                if(!string.IsNullOrEmpty(newFileName))
                {
                    sdlg.InitialDirectory = Path.GetDirectoryName(newFileName);
                    sdlg.FileName = Path.GetFileName(newFileName);

                }
                else
                {
                    sdlg.FileName = this.fileName;
                }

                DialogResult res = sdlg.ShowDialog();
                if (res != DialogResult.OK) return false;
                newFileName = sdlg.FileName;

                try
                {
                    using (StreamWriter sw = new StreamWriter(newFileName))
                    {
                        int r = tempMaze.GetUpperBound(0) + 1;
                        int c = tempMaze.GetUpperBound(tempMaze.Rank - 1) + 1;
                        sw.WriteLine(r.ToString());
                        sw.WriteLine(c.ToString());
                        //enter the maze data

                        string tempString = "";

                        //MessageBox.Show(r.ToString());

                        
                        for (int i = 0; i < r; i++)
                        {
                            for (int j = 0; j < c; j++)
                            {
                                tempString += tempMaze[i, j].ToString() + " ";
                            }
                            sw.WriteLine(tempString);
                            tempString = "";
                        }
                        //sw.WriteLine(rows.ToString() + " " + columns.ToString());
                        ////enter the number of moves that were used
                        //sw.WriteLine(moveCount);
                        ////enter the move list which is stored in as strings
                        ////sw.WriteLine(dataString);
                        //for (int i = 0; i < moveCount; i++)
                        //{
                        //    sw.WriteLine(dataString[i]);
                        //}
                        //sw.WriteLine(ptCount.ToString());
                        //for (int i = 0; i < ptCount; i++)
                        //    sw.WriteLine(myPoints[i].X.ToString() + " "
                        //                        + myPoints[i].Y.ToString());
                    }
                }
                catch (Exception e)
                {
                    MessageBox.Show("Can't save file: " + newFileName + "\r\n" + e.Message, Application.ProductName);
                    return false;
                }
                return true;
        }//End SaveWork


        //***************************MoveLocation**********************
        // This is the driver function for the MoveLocationWork
        // function which will move the current position cursor to
        // a set location.
        //*************************************************************
        public bool MoveLocation(int x, int y)
        {
            return MoveLocationWork(x,y);
        }

        //*************************MoveLocationWork************************
        // This is the work function for Move Location. It will move
        // the current position marker to a specified location in the
        // maze. It will allow movement onto walls but not outside the
        // boundary of the maze. So it will not be able to move to (-1,3).
        // This function will return a bool in order to make sure that the
        // coordinates are within the correct boundary of the maze. If so
        // then true is returned. Else False.
        //*****************************************************************
        private bool MoveLocationWork(int x, int y)
        {
            //Check and make sure that the location is in bounds.
            //(This should have been done at the maze but will check again)
            if (x >= 0 && x < rows && y >= 0 && y < columns)
            {
                moveData = "From: (" + curXLoc.ToString() + ", " + curYLoc.ToString() + ") To: (" + x.ToString() + ", " + y.ToString() + ") ";
                //Good Data so set the values
                if (mazeBoard[curXLoc, curYLoc] == FINISHCURRENTCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = FINISHCOLOR;
                }
                else if (mazeBoard[curXLoc, curYLoc] != ERRORCURRENTLOCCOLOR && mazeBoard[curXLoc, curYLoc] != WALLCOLOR)
                {
                    //Not a wall
                    mazeBoard[curXLoc, curYLoc] = BEENPATHCOLOR;
                }
                else
                {
                    
                    mazeBoard[curXLoc, curYLoc] = ERRORCOLOR;
                }
                if (mazeBoard[x, y] == WALLCOLOR || mazeBoard[x, y] == ERRORCOLOR)
                {
                    //Heading into a wall
                    moveData += "Wall";
                    mazeBoard[x, y] = ERRORCURRENTLOCCOLOR;
                }
                else if (mazeBoard[x, y] == FINISHCOLOR)
                {
                    moveData += "Finished";
                    mazeBoard[x, y] = FINISHCURRENTCOLOR;
                }
                else
                {
                    //Passable area
                    moveData += "Path";
                    mazeBoard[x, y] = CURRENTLOCCOLOR;
                }
                curXLoc = x;
                curYLoc = y;

                if (UpdateMoveLog != null)
                {
                    this.UpdateMoveLog(this, EventArgs.Empty);
                }

                return true;
            }
            else
            {
                return false;
            }
        }

        //*****************************MoveUp**************************
        // This function will call the MoveUpWork function in order
        // to move the items in the maze.
        //*************************************************************
        public void MoveUp()
        {
            MoveUpWork();
        }

        //***************************MoveUpWork*************************
        // This is the worker of the MoveUp function. It will move the
        // current position marker in the maze up one unit. This will
        // allow movement through walls in order to show the steps
        // taken. However it will not allow the marker to move outside
        // the set boundary of the maze. So it will not move to (-1,3).
        //**************************************************************
        private void MoveUpWork()
        {
            if (curXLoc - 1 < 0)
            {
                //Error Out of bounds
                MessageBox.Show("The current location would be outside of the maze boundary. Nothing was changed.");
            }
            else
            {
                //Is good
                
                if (mazeBoard[curXLoc, curYLoc] == ERRORCURRENTLOCCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = ERRORCOLOR;
                }
                else if (mazeBoard[curXLoc, curYLoc] == FINISHCURRENTCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = FINISHCOLOR;
                }
                else
                {
                    mazeBoard[curXLoc, curYLoc] = BEENPATHCOLOR;
                }

                moveData = "From: (" + curXLoc.ToString() + ", " + curYLoc.ToString() + ") To: (" + (curXLoc - 1).ToString() + ", " + curYLoc.ToString() + ") ";

                if (mazeBoard[curXLoc - 1, curYLoc] == WALLCOLOR || mazeBoard[curXLoc - 1, curYLoc] == ERRORCOLOR)
                {
                    //Then is a wall
                    moveData += "Wall";
                    mazeBoard[curXLoc - 1, curYLoc] = ERRORCURRENTLOCCOLOR;
                    curXLoc--;
                }
                else if (mazeBoard[curXLoc - 1, curYLoc] == FINISHCOLOR || mazeBoard[curXLoc - 1, curYLoc] == FINISHCURRENTCOLOR)
                {
                    //Then is the Finish Line
                    moveData += "Finished";
                    mazeBoard[curXLoc - 1, curYLoc] = FINISHCURRENTCOLOR;
                    curXLoc--;
                }
                else
                {
                    moveData += "Path";
                    mazeBoard[curXLoc - 1, curYLoc] = CURRENTLOCCOLOR;
                    curXLoc--;
                }

                if (UpdateMoveLog != null)
                {
                    this.UpdateMoveLog(this, EventArgs.Empty);
                }

            }

        }//End of MoveUpWork

        //*************************MoveDown**************************
        // This function will be used to call the MoveDownWork
        // funcion in order to move the current position down
        // one unit.
        //***********************************************************
        public void MoveDown()
        {
            MoveDownWork();
        }//End of MoveDown

        //***********************MoveDownWork**************************
        // This function will provide the work for the move down
        // function. It will move the current position marker down
        // one unit. It will allow movement through wall but not
        // outside of the maze boundaries. So it will not allow
        // movement to (8,3) in a maze that is 7x3. It will also
        // update the movement log in the main form.
        //*************************************************************
        private void MoveDownWork()
        {
            if (curXLoc + 1 > rows-1)
            {
                //Error Out of bounds
                MessageBox.Show("The current location would be outside of the maze boundary. Nothing was changed.");
            }
            else
            {
                //Is good
                if (mazeBoard[curXLoc, curYLoc] == ERRORCURRENTLOCCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = ERRORCOLOR;
                }
                else if (mazeBoard[curXLoc, curYLoc] == FINISHCURRENTCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = FINISHCOLOR;
                }
                else
                {
                    mazeBoard[curXLoc, curYLoc] = BEENPATHCOLOR;
                }

                moveData = "From: (" + curXLoc.ToString() + ", " + curYLoc.ToString() + ") To: (" + (curXLoc + 1).ToString() + ", " + curYLoc.ToString() + ") ";

                if (mazeBoard[curXLoc + 1, curYLoc] == WALLCOLOR || mazeBoard[curXLoc + 1, curYLoc] == ERRORCOLOR)
                {
                    //Then is a wall
                    moveData += "Wall";
                    mazeBoard[curXLoc + 1, curYLoc] = ERRORCURRENTLOCCOLOR;
                    curXLoc++;
                }
                else if (mazeBoard[curXLoc + 1, curYLoc] == FINISHCOLOR || mazeBoard[curXLoc + 1, curYLoc] == FINISHCURRENTCOLOR)
                {
                    //Then is the Finish Line
                    moveData += "Finished";
                    mazeBoard[curXLoc + 1, curYLoc] = FINISHCURRENTCOLOR;
                    curXLoc++;
                }
                else
                {
                    moveData += "Path";
                    mazeBoard[curXLoc + 1, curYLoc] = CURRENTLOCCOLOR;
                    curXLoc++;
                }

                if (UpdateMoveLog != null)
                {
                    this.UpdateMoveLog(this, EventArgs.Empty);
                }

            }
        }//End of MoveDownWork


        //*****************************MoveLeft**************************
        // This function will call the move left work function in order
        // to move the current position marker left one unit.
        //***************************************************************
        public void MoveLeft()
        {
            MoveLeftWork();
        }//End of MoveLeft

        //***************************MoveLeftWork*************************
        // This will move the current position marker left one unit. It
        // will allow movement through walls but not outisde the boundary
        // of the maze. So it will not allow movement to (3,-1) since the
        // lowest value of the mazes columns is 0 no negative value would
        // be accepted.
        //****************************************************************
        private void MoveLeftWork()
        {
            if (curYLoc - 1 < 0)
            {
                //Error Out of bounds
                MessageBox.Show("The current location would be outside of the maze boundary. Nothing was changed.");
            }
            else
            {
                //Is good
                if (mazeBoard[curXLoc, curYLoc] == ERRORCURRENTLOCCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = ERRORCOLOR;
                }
                else if (mazeBoard[curXLoc, curYLoc] == FINISHCURRENTCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = FINISHCOLOR;
                }
                else
                {
                    mazeBoard[curXLoc, curYLoc] = BEENPATHCOLOR;
                }

                moveData = "From: (" + curXLoc.ToString() + ", " + curYLoc.ToString() + ") To: (" + curXLoc.ToString() + ", " + (curYLoc - 1).ToString() + ") ";


                if (mazeBoard[curXLoc, curYLoc-1] == WALLCOLOR || mazeBoard[curXLoc, curYLoc-1] == ERRORCOLOR)
                {
                    //Then is a wall
                    moveData += "Wall";
                    mazeBoard[curXLoc, curYLoc-1] = ERRORCURRENTLOCCOLOR;
                    curYLoc--;
                }
                else if (mazeBoard[curXLoc, curYLoc - 1] == FINISHCOLOR || mazeBoard[curXLoc, curYLoc - 1] == FINISHCURRENTCOLOR)
                {
                    //Then is the Finish Line
                    moveData += "Finished";
                    mazeBoard[curXLoc, curYLoc - 1] = FINISHCURRENTCOLOR;
                    curYLoc--;
                }
                else
                {
                    moveData += "Path";
                    mazeBoard[curXLoc, curYLoc-1] = CURRENTLOCCOLOR;
                    curYLoc--;
                }

                if (UpdateMoveLog != null)
                {
                    this.UpdateMoveLog(this, EventArgs.Empty);
                }

            }
        }//End of MoveLeftWork

        //***************************MoveRight***************************
        // This fucntion will call the MoveRightWork function in order
        // to move right one unit.
        //***************************************************************
        public void MoveRight()
        {
            MoveRightWork();
        }//End of MoveRight

        //***************************MoveRightWork************************
        // This will move the current position marker right one unit. It
        // will allow movement through walls but not outisde the boundary
        // of the maze. So it will not allow movement to (3,8) if the
        // max column for that maze was 7.
        //****************************************************************
        private void MoveRightWork()
        {
            if (curYLoc + 1 > columns-1)
            {
                //Error Out of bounds
                MessageBox.Show("The current location would be outside of the maze boundary. Nothing was changed.");
            }
            else
            {
                
                //Is good
                if (mazeBoard[curXLoc, curYLoc] == ERRORCURRENTLOCCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = ERRORCOLOR;
                }
                else if (mazeBoard[curXLoc, curYLoc] == FINISHCURRENTCOLOR)
                {
                    mazeBoard[curXLoc, curYLoc] = FINISHCOLOR;
                }
                else
                {
                    mazeBoard[curXLoc, curYLoc] = BEENPATHCOLOR;
                }

                moveData = "From: (" + curXLoc.ToString() + ", " + curYLoc.ToString() + ") To: (" + curXLoc.ToString() + ", " + (curYLoc + 1).ToString() + ") ";

                if (mazeBoard[curXLoc, curYLoc + 1] == WALLCOLOR || mazeBoard[curXLoc, curYLoc + 1] == ERRORCOLOR)
                {
                    //Then is a wall
                    moveData += "Wall";
                    mazeBoard[curXLoc, curYLoc + 1] = ERRORCURRENTLOCCOLOR;
                    curYLoc++;
                }
                else if (mazeBoard[curXLoc, curYLoc + 1] == FINISHCOLOR || mazeBoard[curXLoc, curYLoc + 1] == FINISHCURRENTCOLOR)
                {
                    //Then is the Finish Line
                    moveData += "Finished";
                    mazeBoard[curXLoc, curYLoc + 1] = FINISHCURRENTCOLOR;
                    curYLoc++;
                }
                else
                {
                    moveData += "Path";
                    mazeBoard[curXLoc, curYLoc + 1] = CURRENTLOCCOLOR;
                    curYLoc++;
                }

                if (UpdateMoveLog != null)
                {
                    this.UpdateMoveLog(this, EventArgs.Empty);
                }

            }
        }//End of MoveRightWork

        //*********************************CheckWallUp***************************
        // This function will check and see if there is a wall one unit
        // Up from the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallUp()
        {
            int tempX = curXLoc;
            int tempY = curYLoc;

            tempX--;
            if (tempX < 0 || tempX > rows - 1)
            {
                return -1;
            }
            else if (tempY < 0 || tempY > columns - 1)
            {
                return -1;
            }
            else
            {
                if (mazeBoard[tempX, tempY] == WALLCOLOR)
                {
                    return 0;
                }
                else
                {
                    return 1;

                }
            }
        }//End of CheckWallUp

        //*********************************CheckWallDown***********************
        // This function will check and see if there is a wall one unit
        // down from the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallDown()
        {
            int tempX = curXLoc;
            int tempY = curYLoc;

            tempX++;
            if (tempX < 0 || tempX > rows - 1)
            {
                return -1;
            }
            else if (tempY < 0 || tempY > columns - 1)
            {
                return -1;
            }
            else
            {
                if (mazeBoard[tempX, tempY] == WALLCOLOR)
                {
                    return 0;
                }
                else
                {
                    return 1;

                }
            }
        }//End of CheckWallDown

        //*********************************CheckWallLeft***********************
        // This function will check and see if there is a wall one unit
        // Left of the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallLeft()
        {
            int tempX = curXLoc;
            int tempY = curYLoc;

            tempY--;
            if (tempX < 0 || tempX > rows - 1)
            {
                return -1;
            }
            else if (tempY < 0 || tempY > columns - 1)
            {
                return -1;
            }
            else
            {
                if (mazeBoard[tempX, tempY] == WALLCOLOR)
                {
                    return 0;
                }
                else
                {
                    return 1;

                }
            }
        }//End of CheckWallLeft

        //*********************************CheckWallRight***********************
        // This function will check and see if there is a wall one unit
        // Right from the current position. If there is then it will return a
        // 0 if not it will return a 1. If the iem is outside of the maze
        // boundary it will return a -1.
        //**********************************************************************
        public int CheckWallRight()
        {
            int tempX = curXLoc;
            int tempY = curYLoc;

            tempY++;
            if (tempX < 0 || tempX > rows - 1)
            {
                return -1;
            }
            else if (tempY < 0 || tempY > columns - 1)
            {
                return -1;
            }
            else
            {
                if (mazeBoard[tempX, tempY] == WALLCOLOR)
                {
                    return 0;   //IS A WALL
                }
                else
                {
                    return 1;

                }
            }
        }//End of CheckWallRight

        //*****************************CheckPathUp*************************
        // This function will call the CheckPathUpWork function in order
        // to check and see what type of square is one unit up from the
        // current position.
        //*****************************************************************
        public int CheckPathUp()
        {
            return CheckPathUpWork();
        }//End of CheckPathUp

        //**********************************CheckPathUpWork**********************
        // This function will check and see what the value of the maze square
        // one unit up from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //************************************************************************
        private int CheckPathUpWork()
        {
            if (CheckWallUp() == 0)
            {
                return 0;   //Means it is a wall
            }
            else
            {
                int tempX = curXLoc;
                tempX--;
                if (tempX < 0 || tempX > rows - 1)
                {
                    return -1;
                }
                else if (mazeBoard[tempX, curYLoc] == BEENPATHCOLOR)
                {
                    return 1;   //Been Path
                }
                else
                {
                    return 2;   //Not Yet Been
                }
            }
        }//End of CheckPathUpWork

        //*****************************CheckPathDown*************************
        // This function will call the CheckPathDownWork function in order
        // to check and see what type of square is one unit Down from the
        // current position.
        //********************************************************************
        public int CheckPathDown()
        {
            return CheckPathDownWork();
        }//End of CheckPathDown

        //**********************************CheckPathDownWork**********************
        // This function will check and see what the value of the maze square
        // one unit down from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //**************************************************************************
        private int CheckPathDownWork()
        {
            if (CheckWallDown() == 0)
            {
                return 0;   //Means it is a wall
            }
            else
            {
                int tempX = curXLoc;
                tempX++;
                if (tempX < 0 || tempX > rows - 1)
                {
                    return -1;
                }
                else if (mazeBoard[tempX, curYLoc] == BEENPATHCOLOR)
                {
                    return 1;   //Been Path
                }
                else
                {
                    return 2;   //Not Yet Been
                }
            }
        }//End of CheckPathDownWork

        //*****************************CheckPathLeft**************************
        // This function will call the CheckPathLeftWork function in order
        // to check and see what type of square is one unit Left from the
        // current position.
        //********************************************************************
        public int CheckPathLeft()
        {
            return CheckPathLeftWork();
        }//End of CheckPathLeft

        //**********************************CheckPathLeftWork**********************
        // This function will check and see what the value of the maze square
        // one unit left from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //**************************************************************************
        private int CheckPathLeftWork()
        {
            if (CheckWallLeft() == 0)
            {
                return 0;   //Means it is a wall
            }
            else
            {
                int tempY = curYLoc;
                tempY--;
                if (tempY < 0 || tempY > columns - 1)
                {
                return -1;
                }
                else if (mazeBoard[curXLoc, tempY] == BEENPATHCOLOR)
                {
                    return 1;   //Been Path
                }
                else
                {
                    return 2;   //Not Yet Been
                }
            }
        }//End of CheckPathLeftWork

        //*****************************CheckPathRight*************************
        // This function will call the CheckPathRightWork function in order
        // to check and see what type of square is one unit Right from the
        // current position.
        //********************************************************************
        public int CheckPathRight()
        {
            return CheckPathRightWork();
        }//End of CheckPathRight

        //**********************************CheckPathRightWork**********************
        // This function will check and see what the value of the maze square
        // one unit right from the current position is. It will return a 0 if
        // it is a wall, a -1 if it is out of bounds, a 1 if it is marked as
        // having been to that square, or a 2 if the cursor has not yest moved
        // to that square.
        //**************************************************************************
        private int CheckPathRightWork()
        {
            if (CheckWallRight() == 0)
            {
                return 0;   //Means it is a wall
            }
            else
            {
                int tempY = curYLoc;
                tempY++;
                if (tempY < 0 || tempY > columns - 1)
                {
                    return -1;
                }
                else if (mazeBoard[curXLoc, tempY] == BEENPATHCOLOR)
                {
                    return 1;   //Been Path
                }
                else
                {
                    return 2;   //Not Yet Been
                }
            }
        }//End of CheckPathRightWork

        //Return -1 for outside maze or error, 0 for wall, 1 for path
        //********************************CheckWall************************
        // This function is supposed to be a generic way to check and see
        // if there is a wall in the direction that is sent as a parameter
        // however as it is currently it will not work.
        //*****************************************************************
        public int CheckWall(int direction)
        {
            int tempX = curXLoc;
            int tempY = curYLoc;

            //if (direction == "up" || direction == "Up")
            //{
                
            //    //if (mazeBoard[(curXLoc - 1), curYLoc] == WALLCOLOR)
            //    //{
            //    //    return 0;
            //    //}
            //    //else
            //    //{
            //    //    return 1;
            //    //}
            //}
            switch (direction)
            {
                
                case 0: //Up
                    tempX--;
                    break;
                    //if (mazeBoard[(curXLoc - 1), curYLoc] == WALLCOLOR)
                    //{
                    //    return 0;
                    //}
                    //else
                    //{
                    //    return 1;

                    //}
                case 1: //Down
                    tempX++;
                    break;
                    //if (mazeBoard[(curXLoc + 1), curYLoc] == WALLCOLOR)
                    //{
                    //    return 0;
                    //}
                    //else
                    //{
                    //    return 1;

                    //}
                case 2: //Left
                    tempY--;
                    break;
                    //if (mazeBoard[curXLoc, (curYLoc - 1)] == WALLCOLOR)
                    //{
                    //    return 0;
                    //}
                    //else
                    //{
                    //    return 1;

                    //}
                case 3: //right
                    tempY++;
                    break;
                    //if (mazeBoard[curXLoc, (curYLoc + 1)] == WALLCOLOR)
                    //{
                    //    return 0;
                    //}
                    //else
                    //{
                    //    return 1;

                    //}
                default:
                    MessageBox.Show("Invalid direction. Please enter up, down, left, or right");
                    return -1;
            }

            if (tempX < 0 || tempX > rows - 1)
            {
                return -1;
            }
            else if (tempY < 0 || tempY > columns - 1)
            {
                return -1;
            }
            else
            {
                if (mazeBoard[tempX, tempY] == WALLCOLOR)
                {
                    return 0;
                }
                else
                {
                    return 1;

                }
            }


        }//End of Wall Check
                   
        //*******************************CheckEnd******************************
        // This function will call the CheckEndWork function in order to see
        // if the end of the maze has been reached or not.
        //*********************************************************************
        public int CheckEnd()
        {
            return CheckEndWork();
        }

        //*******************************CheckEndWork***************************
        // This function will check and see if the end of the maze has
        // been reached and will return a 1 if yes or a 0 if no.
        //**********************************************************************
        private int CheckEndWork()
        {
            if (curXLoc == finishXLoc && curYLoc == finishYLoc)
            {
                return 1;
            }
            else
            {
                return 0;
            }
        }

    }//End of MazeDoc Class


}

