﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace RecursiveMazeMain
{
    public partial class NewMazeForm : Form
    {
        private int newMazeRow;
        private int newMazeColumn;
        private Button[,] newMazeBoard;
        private int[,] newMazeBoardValues;
        private int startAmount = 0;
        private int finishAmount = 0;

        private const int BUTTONSIZE = 50;
        private Color WALL = Color.Gray;    //0
        private Color PATH = Color.LightGray;   //1
        private Color START = Color.Yellow; //5
        private Color FINISH = Color.RoyalBlue; //6

        public int NewMazeRow
        {
            get
            {
                return newMazeRow;
            }
            set
            {
                newMazeRow = value;
            }
        }

        public int NewMazeColumn
        {
            get
            {
                return newMazeColumn;
            }
            set
            {
                newMazeColumn = value;
            }
        }

        //public int[,] NewMazeBoardValues
        //{
        //    get
        //    {
        //        return NewMazeBoardValues;
        //    }
        //}

        public NewMazeForm()
        {
            InitializeComponent();
            //InitializeButtons();
            //MessageBox.Show(newMazeColumn.ToString());
            //MessageBox.Show(newMazeRow.ToString());
            //MessageBox.Show("HERE");
            //Button test = new Button();
            //test.Size = new Size(BUTTONSIZE, BUTTONSIZE);
            //test.Location = new Point(100, 100);
            //this.Controls.Add(test);
        }

        //*********************************StartButtons****************************
        // This function is used to call the Initialize Buttons function.
        //*************************************************************************
        public void StartButtons()
        {
            InitializeButtons();
        }//End of StartButtons

        //*************************************InitialiazeButtons*********************
        // This function will display all of the buttons used to create the maze
        // by using the data that was obtained from the new maze start form.
        //****************************************************************************
        private void InitializeButtons()
        {
            //Create and display the buttons for the new maze
            if (newMazeBoard != null)
            {
                foreach (Button bn in newMazeBoard)
                {
                    bn.Hide();
                }
            }
            newMazeBoard = new Button[newMazeRow, newMazeColumn];
            int r = newMazeBoard.GetUpperBound(0) + 1;
            int c = newMazeBoard.GetUpperBound(newMazeBoard.Rank - 1) + 1;
            int xloc = 50;
            int yloc = 50;
            for (int i = 0; i < r; i++)
            {
                for (int j = 0; j < c; j++)
                {
                    newMazeBoard[i, j] = new Button();
                    newMazeBoard[i, j].Size = new Size(BUTTONSIZE, BUTTONSIZE);
                    newMazeBoard[i, j].Location = new Point(xloc, yloc);
                    newMazeBoard[i, j].BackColor = WALL;
                    //assign the click event
                    newMazeBoard[i, j].Click += new EventHandler(MazeButtonClick);
                    newMazeBoardValues = new int[newMazeRow, newMazeColumn];
                    newMazeBoardValues[i, j] = 0;
                    
                    xloc += 50;
                }
                xloc = 50;
                yloc += 50;
            }
            //MessageBox.Show("HEREHERE");
            for (int i = 0; i < r; i++)
            {
                for (int j = 0; j < c; j++)
                {
                    this.Controls.Add(newMazeBoard[i, j]);
                }
            }
           //Invalidate();
        }//End of Initialize Buttons

        //*******************************ObtainNewMaze*****************************
        // This function will be used to retrieve the maze from the new maze form
        // and will return the result of the worker function.
        //*************************************************************************
        public int[,] ObtainNewMaze()
        {
            return ObtainNewMazeWork();
        }//End of ObtainNewMaze

        //**********************************ObtainNewMazeWork*************************
        // This functon will provide then work for the function ObtainNewMaze. It will
        // return the values of the created maze based on the colors of the buttons
        // in the maze.
        //****************************************************************************
        private int[,] ObtainNewMazeWork()
        {
            for (int i = 0; i < newMazeRow; i++)
            {
                for (int j = 0; j < newMazeColumn; j++)
                {
                    if (newMazeBoard[i, j].BackColor == WALL)
                    {
                        newMazeBoardValues[i, j] = 0;
                    }
                    else if (newMazeBoard[i, j].BackColor == PATH)
                    {
                        newMazeBoardValues[i, j] = 1;
                    }
                    else if (newMazeBoard[i, j].BackColor == START)
                    {
                        newMazeBoardValues[i, j] = 5;
                    }
                    else if (newMazeBoard[i, j].BackColor == FINISH)
                    {
                        
                        newMazeBoardValues[i, j] = 6;
                    }
                }
            }
            return newMazeBoardValues;
        }

        //********************************MazeButtonClick**************************
        // This event will run when the maze buttons are clicked. It will change
        // the color of the buttons based on the amount of clicks. It will then
        // alter the amount of start and finish areas so that a maze can and must
        // have only one of each.
        //*************************************************************************
        private void MazeButtonClick(object sender, EventArgs e)
        {
            Button btn = (Button)sender;


            //bool temp;

            if (btn.BackColor == WALL)
            {
                btn.BackColor = PATH;                
            }
            else if (btn.BackColor == PATH)
            {
                startAmount++;
                btn.BackColor = START;
            }
            else if (btn.BackColor == START)
            {
                if (startAmount > 0)
                {
                    startAmount--;
                }
                finishAmount++;
                btn.BackColor = FINISH;
            }
            else
            {
                if (finishAmount > 0)
                {
                    finishAmount--;
                }
                btn.BackColor = WALL;
            }
        }//End of button click

        //*********************************SaveButtonClick***********************
        // This will be called when the save button is clicked. It will first
        // make sure that there is only on start and finish in the maze. It will
        // then return an OK as the dialog result.
        //***********************************************************************
        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (startAmount == 1 && finishAmount == 1)
            {
                DialogResult = DialogResult.OK;
            }
            else
            {
                MessageBox.Show("Please make sure that there is one Start and Finish");
                MessageBox.Show(finishAmount.ToString());
                MessageBox.Show(startAmount.ToString());
            }
        }

        //********************************CancelButtonClick************************
        // This event will be called when the cancel button is clicked. It will 
        // return a dialog result of cancel.
        //*************************************************************************
        private void CancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }
    }
}
