﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.IO.Pipes;
using System.Threading;

//Close the pipeStreamWrite when the form closes
//Open the pipeStreamWrite when the form is created


/******************************Need to create*****************************
- Move tracker so that user can see past moves and track the recursion
- Create Status Bar at bottom to show current location in x and y coord
- Also do the New, Save, and Save As features
- The Undo feature can also be implemented somewhere
- Finish implementing the help menu
**************************************************************************/

/***********************************Notes********************************
- Might simply create a function that writes the string parameter to
  a file for the server to read and then reuse it when needed since they
  will all be written to the same file.
- ALL files will be written to and read from the RecursiveMazeMain folder
  area since it should not matter where the user creates their folder or
  what they name it.
*************************************************************************/


namespace RecursiveMazeMain
{
    public partial class RecursiveMazeMainForm : Form
    {

        //Need items for the maze squares such as color

        private Color WALL = Color.Gray;    //0 in File
        private Color CURRENT = Color.Gold; //Not in File
        private Color ERRORSPACE = Color.Red;   //Not in File
        private Color BEENTO = Color.Green; //Not in File
        private Color PASSABLE = Color.LightGray;   //1 in File
        private Color START = Color.Yellow; //5 in File
        private Color FINISH = Color.RoyalBlue; //6 in File
        private Color FINISHCURRENT = Color.Blue;   //Not in File
        private Color ERRORCURRENT = Color.Orange;  //7 in File

        //Constants
        private const int SQUARESIZE = 50;      //Height and Width of maze squares
        private const int STARTLOCATIONX = 300; //x coordinate to start maze
        private const int STARTLOCATIONY = 0;  //y coordinate to start maze

        //This will be used to receive data from the server
        private const string PIPEREADING = "rmgPipeServer";
        NamedPipeClientStream pipeStreamRead = new NamedPipeClientStream(PIPEREADING);
        //StreamWriter sw = new StreamWriter(pipeStreamWrite);

        //This will be used to send data to the Server
        private const string PIPESENDING = "rmgPipeClient";
        System.IO.Pipes.NamedPipeServerStream pipeStreamWrite = new NamedPipeServerStream(PIPESENDING);

        //Create the thread
        private Thread pipeThread;
        private int sleepTime = 100;

        private bool threadOn = false;

        private MazeDoc mazeData;
        private ListBox moveLBox;


        public RecursiveMazeMainForm()
        {
            InitializeComponent();
            mazeData = new MazeDoc();

            //this.mazeData.Open += new System.EventHandler(OpenDocument);
            this.mazeData.UpdateMoveLog += new System.EventHandler(UpdateMoveLog);

        }

        //**************************OpenMenuClick****************************
        // When the open menu item is clicked it will display the open
        // dialog in which the user can then open a maze that has already
        // been created. This will only open files with the extension rmg.
        //*******************************************************************
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool openCheck = mazeData.Open();
            if (openCheck)
            {
                //Hide intro label and display the maze
                if (IntroLabel.Visible)
                {
                    IntroLabel.Hide();
                }
                if (MazeRowTextBox.Visible == false)
                {
                    MazeRowTextBox.Show();
                    MazeColumnTextBox.Show();
                    MoveMazeCursorButton.Show();
                    RowLabel.Show();
                    ColumnLabel.Show();
                    StartConnectionButton.Show();
                    ManualMovementButton.Show();
                    ThreadControlCheckBox.Show();

                    //finish creating and showing the list box
                    moveLBox = new ListBox();
                    moveLBox.Location = new Point(65, 50);
                    moveLBox.Width = 150;
                    moveLBox.Height = 250;
                    Controls.Add(moveLBox);

                    Label lBoxLabel = new Label();
                    lBoxLabel.Width = 280;
                    lBoxLabel.Text = "Move List (row number, column number)";
                    lBoxLabel.Location = new Point(65, 35);
                    Controls.Add(lBoxLabel);

                    //Display the row and column labels
                    Label rowInfoLabel = new Label();
                    rowInfoLabel.Width = 55;
                    rowInfoLabel.Text = "Row 0:";
                    rowInfoLabel.Location = new Point(290, 70);
                    Controls.Add(rowInfoLabel);

                    Label columnInfoLabel = new Label();
                    columnInfoLabel.Width = 55;
                    columnInfoLabel.Text = "Column 0:";
                    columnInfoLabel.Location = new Point(345, 27);
                    Controls.Add(columnInfoLabel);
                }
                Invalidate();
            }
        }

        //*********************************newToolStripMenuItem**************************
        // This function will display the new maze creation forms and then save the
        // newly created maze if the user desires. The save is done by calling the save
        // function from mazeDoc. This function will also create new instances of the
        // NewMazeStartForm and the NewMazeForm and will then display them.
        //*******************************************************************************
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Call the new maze form which will enable the user to create
            //a new maze either through randomness or by user controls
            NewMazeStartForm NMForm = new NewMazeStartForm();
            NMForm.ShowDialog();
            if (NMForm.DialogResult == DialogResult.OK)
            {
                int newRow = NMForm.NewRows;
                int newCol = NMForm.NewColumns;
                //MessageBox.Show(newRow.ToString());
                NewMazeForm NMForm2 = new NewMazeForm();
                NMForm2.NewMazeRow = newRow;
                NMForm2.NewMazeColumn = newCol;
                NMForm2.StartButtons();
                MessageBox.Show(NMForm2.NewMazeRow.ToString());
                NMForm2.ShowDialog();
                if(NMForm2.DialogResult == DialogResult.OK)
                {
                    //obtain the data
                    int[,] tempMaze = new int[NMForm2.NewMazeRow, NMForm2.NewMazeColumn];
                    tempMaze = NMForm2.ObtainNewMaze();
                    mazeData.Save(tempMaze, NMForm2.NewMazeRow, NMForm2.NewMazeColumn);
                }

            }

        }

        //*******************************HowToPlayMenuClick*****************************
        // This function will be called whenever the how to play menu item is clicked.
        // It will then create a new instance of the help form and then display it.
        //******************************************************************************
        private void HowToPlayToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MazeHelpForm mazeHelp = new MazeHelpForm();
            mazeHelp.Show();
        }



        //This function is mostly used to test the creation of a solid
        //colored rectangle and testing the colors
        private void FillRectangleInt(object sender, EventArgs e)
        {

            // Create solid brush.
            SolidBrush blueBrush = new SolidBrush(Color.Blue);
            SolidBrush testing = new SolidBrush(Color.LightGray);
            Graphics t1 = this.CreateGraphics();
            // Create location and size of rectangle. 
            int x = 350;
            int y = 50;
            int width = 50;
            int height = 50;
            IntroLabel.Hide();
            t1.FillRectangle(testing, x, y, width, height);
            testing.Color = Color.Gray;
            t1.FillRectangle(testing, x + width, y, width, height);

            // Fill rectangle to screen.
            //e.Graphics.FillRectangle(blueBrush, x, y, width, height);
        }//End of FillRectangleInt

        //***********************************mazePrintScreen*****************************
        // This event will be called when paint event for the form is called. It will
        // display the maze if it has been opened and created in the mazDoc and will
        // update the maze when changes are made. It will also create a thread that will
        // check and see if the program has recieved any information from the server.
        //*******************************************************************************
        private void mazePrintScreen(object sender, PaintEventArgs e)
        {
            if (mazeData.MazeCreated)   //equals true
            {
                if (ThreadControlCheckBox.Checked == true)
                {
                    if (ManualMovementButton.Enabled != false)
                    {
                        ManualMovementButton.Enabled = false;
                    }
                    pipeThread = new Thread(new ThreadStart(PipeThreadWork));
                    pipeThread.Start();
                }
                else
                {
                    //pipeThread.Abort();
                    ManualMovementButton.Enabled = true;
                }
                //first create the graphics and brush in order to create the maze with
                Graphics mazeGraphics = this.CreateGraphics();
                SolidBrush mazeColoror = new SolidBrush(Color.Gray);

                //paint the maze on the screen
                for (int i = 0; i < mazeData.Rows; i++) //In charge of the rows
                {

                    for (int j = 0; j < mazeData.Columns; j++)  //In charge of the columns
                    {
                        //Obtain the color
                        switch (mazeData.MazeBoard[i, j])
                        {
                            case 0:
                                //Wall
                                mazeColoror.Color = Color.Gray;
                                break;
                            case 1:
                                mazeColoror.Color = Color.LightGray;
                                break;
                            case 2:
                                mazeColoror.Color = Color.Green;
                                break;
                            case 3:
                                mazeColoror.Color = Color.Red;
                                break;
                            case 4:
                                mazeColoror.Color = Color.Gold;
                                break;
                            case 5:
                                mazeColoror.Color = Color.Yellow;
                                break;
                            case 6:
                                mazeColoror.Color = Color.RoyalBlue;
                                break;
                            case 7:
                                mazeColoror.Color = Color.Orange;
                                break;
                            case 8:
                                mazeColoror.Color = Color.Blue;
                                break;
                            default:
                                mazeColoror.Color = Color.LightGray;
                                break;
                        }//end of Swich case

                        //Draw the maze section
                        mazeGraphics.FillRectangle(mazeColoror, STARTLOCATIONX + (SQUARESIZE * (j + 1)), STARTLOCATIONY + (SQUARESIZE * (i + 1)), SQUARESIZE, SQUARESIZE);
                    }

                }
            }
            //else don't
        }//End of mazePrintScreen

        //********************************moveMazeCursorButtonClick**********************
        // This event will run whenever the move cursor button is clicked. It will first
        // make sure that the row and column text boxes are not empty. If they are it
        // will notify the user and will not change anything in the maze. If there are
        // values then it will call the mazeDoc function for moving to a set location.
        //********************************************************************************
        private void MoveMazeCursorButton_Click(object sender, EventArgs e)
        {
            //Make sure that there are enteries in the text boxes
            bool Test = false;
            try
            {
                if (MazeRowTextBox.Text.Trim() != "")
                {
                    //Then not empty
                    if (MazeColumnTextBox.Text.Trim() != "")
                    {
                        //Not Empty
                        Test = true;
                    }
                    else
                    {
                        MazeColumnTextBox.Focus();
                        throw new Exception();
                    }

                }
                else
                {
                    //Empty
                    MazeRowTextBox.Focus();
                    throw new Exception();
                }
            }
            catch (Exception ex)
            {
                
                MessageBox.Show("Please make sure that the row and column are entered.");
            }


            //move to coordinates on the maze
            if (Test == true)
            {
                int xCoord = Convert.ToInt32(MazeRowTextBox.Text);
                int yCoord = Convert.ToInt32(MazeColumnTextBox.Text);
                if (xCoord < 0 || xCoord > mazeData.Rows || yCoord < 0 || yCoord > mazeData.Columns)
                {
                    //Then Error so display error window
                    MessageBox.Show("Please keep the row and column in the maze.");
                    
                }
                else
                {
                    //The location is in the maze
                    //So Mark the Spot you where at and color current location
                    bool checker = mazeData.MoveLocation(xCoord, yCoord);
                    if(checker == false)
                    {
                        MessageBox.Show("ERROR IN MOVE");
                    }
                    Invalidate();

                }


            }
        }//End of moveCursorLocationClick

        //*********************************StartConnectionClick**********************
        // This event will run when the start connection button is clicked. It will
        // try and connect to the server by calling the start connections function
        // it will then disable the button.
        //***************************************************************************
        private void StartConnectionClick(object sender, EventArgs e)
        {
            //Will Start the connections and the thread that will
            //keep track of the pipes
            StartConnections();
            StartConnectionButton.Enabled = false;

            
        }//End start connection Button Click

        //***************************aboutToolMenuItemClick*************************
        // This event will run when the about menu item is clicked. It will display
        // the about form to the user which holds basic information about the maze.
        //**************************************************************************
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Display the About Form
            AboutForm aboutForm = new AboutForm();
            aboutForm.Show();
        }//End of About Menu Item Click

        //**************************UpdateMoveLogEvent**********************
        // This event will be in charge of updating the move log list box.
        // To do this it will be called from the mazeDoc which is in charge
        // of actually assigning the movement through the maze. This will
        // store the movements in the movement list box.
        //******************************************************************
        private void UpdateMoveLog(object sender, EventArgs e)
        {
            moveLBox.Items.Add(mazeData.MoveData);
        }//End of UpdateMoveLog



        //*******************************SendData*****************************
        // This function will be called when ever something needs to be
        // send over to the Server (other program). It will take a string
        // that will contain the message as a parameter and return nothing.
        //********************************************************************
        private void SendData(string message)
        {
            //string temp = "";
            StreamWriter sw = new StreamWriter(pipeStreamWrite);
            
                sw.AutoFlush = true;
                if (message.Trim() != "")
                {
                    //okay to send
                    sw.WriteLine(message);
                }
                else
                {
                    //no message to send
                    sw.WriteLine("Sorry no message was sent");
                }
            
        }

        //********************************ReadData****************************
        // This function will be used to read the data that is sent over
        // from the server (other program). It will not take any parameters
        // however it will return a string that contains the message that
        // was sent over.
        //*********************************************************************
        private string ReadData()
        {
            string temp = "";
            StreamReader sr = new StreamReader(pipeStreamRead);
            
                if ((temp = sr.ReadLine()) != null)
                {
                    return temp;
                }
                else
                {
                    return "";
                }
            
            //return temp;
        }//End of ReadData


        //*******************************StartConnections*************************
        // This function will be used to set up the connections between the
        // two programs.
        //************************************************************************
        private void StartConnections()
        {
            //Start the thread
            //pipeThread = new Thread(new ThreadStart(PipeThreadWork));
            //pipeThread.Start();

            //Connect the read connection
            pipeStreamRead.Connect(120000); //Will try for up to two minutes
            //MessageBox.Show("READ CONNECTED");

            pipeStreamWrite.WaitForConnection();    //Start the Write server
            //MessageBox.Show("WRITE CONNECTED");


        }//End of StartConnections

        //************************************CornerClose****************************
        // This event will be called when the red X in the top right corer is
        // to close the program.
        //***************************************************************************
        private void CornerClose(object sender, FormClosingEventArgs e)
        {
            if (pipeStreamRead.IsConnected)
            {
                pipeStreamRead.Close();
            }
            if (pipeStreamWrite.IsConnected)
            {
                pipeStreamWrite.WaitForPipeDrain();
                pipeStreamWrite.Close();
            }
            
            
            //if (pipeThread.IsAlive)
            //{
            //    pipeThread.Abort();
            //}
            //Create saves for new mazes
        }//End of Corner Close

        //*********************************PipeThreadWork*****************************
        // This function will be called in order to connect to the pipes and retrieve
        // or send data. It will be the main work function for the thread.
        //****************************************************************************
        void PipeThreadWork()
        {
            //Call the specified function as read from file
            string orderNum = "";  //Might change to a string and split based on , for the last two functions
            string temp = "";
            //int goAhead = -1;
            //possible connect here
            //pipeStreamRead.Connect();
            //then read from the server
            //if (pipeStreamRead. != 0)
            //{
            if (pipeStreamRead.IsConnected)
            {
                orderNum = ReadData();
                // goAhead = 1;
                //}
                //else
                //{
                //    goAhead = -1;
                //    orderNum = "NODATA";
                //}

                //Make generic function for the sending of the data
                switch (orderNum)
                {
                    case "0":
                        break;
                    case "up":
                        //Move Up
                        mazeData.MoveUp();
                        break;
                    case "down":
                        //Move Down
                        mazeData.MoveDown();
                        break;
                    case "left":
                        //Move Left
                        mazeData.MoveLeft();
                        break;
                    case "right":
                        //Move Right
                        mazeData.MoveRight();
                        break;
                    case "moveLoc":
                        //Move Location
                        //Must read the code then the coordinates
                        string[] item = ReadData().Split(',');
                        //int item = Convert.ToInt32(ReadData());
                        //int item2 = Convert.ToInt32(ReadData());
                        mazeData.MoveLocation(Convert.ToInt32(item[1]) ,Convert.ToInt32(item[2]));
                        //mazeData.MoveLocation(
                        break;
                    case "wallCheckNO":
                        //Check Wall
                        //MessageBox.Show("HEHREHRHEHREHR");
                        //Must read the code and then the direction
                         
                        temp = ReadData();
                        MessageBox.Show(temp);
                        SendData((mazeData.CheckWall(Convert.ToInt32(temp)).ToString()));  //get the maze info and return
                        //mazeData.CheckWall(
                        break;
                    case "checkUp":
                        SendData(mazeData.CheckPathUp().ToString());
                        break;
                    case "checkDown":
                        SendData(mazeData.CheckPathDown().ToString());
                        break;
                    case "checkLeft":
                        SendData(mazeData.CheckPathLeft().ToString());
                        break;
                    case "checkRight":
                        SendData(mazeData.CheckPathRight().ToString());
                        break;
                    case "curRow":
                        //Return the current row
                        SendData(mazeData.CurXLoc.ToString());
                        break;
                    case "curCol":
                        //Return the current column
                        SendData(mazeData.CurYLoc.ToString());
                        break;
                    case "endCheck":
                        //Return 1 if end or 0 if not
                        SendData(mazeData.CheckEnd().ToString());
                        break;
                    default:
                        //No a valid code
                        //Send over message saying so
                        string []checkTwice = orderNum.Split(',');
                        if (checkTwice[0] == "moveLoc")
                        {
                            mazeData.MoveLocation(Convert.ToInt32(checkTwice[1]), Convert.ToInt32(checkTwice[2]));
                        }
                        else
                        {

                            SendData("Invalid Code: Please try again.");
                        }
                        break;
                }

                // if (goAhead != -1)
                //{
                Invalidate();
                //}
            }
            Thread.Sleep(sleepTime);
        }

        //**********************************ManualMoveClick**************************
        // This event will be called whenever the Manual Movement Button is clicked.
        // It will move the cursor through the maze one step at a time depending
        // of the directions recieved from the information sent from the server.
        //***************************************************************************
        private void ManualMove_Click(object sender, EventArgs e)
        {
            //Call the specified function as read from file
            string orderNum = "";  //Might change to a string and split based on , for the last two functions
            string temp = "";
            int goAhead = -1;
            //possible connect here
            //pipeStreamRead.Connect();
            //then read from the server
            //if (pipeStreamRead. != 0)
            //{
            orderNum = ReadData();
            goAhead = 1;
            //}
            //else
            //{
            //    goAhead = -1;
            //    orderNum = "NODATA";
            //}

            //Make generic function for the sending of the data
            //switch (orderNum)
            //{
            //    case "0":
            //        break;
            //    case "up":
            //        //Move Up
            //        mazeData.MoveUp();
            //        break;
            //    case "down":
            //        //Move Down
            //        mazeData.MoveDown();
            //        break;
            //    case "left":
            //        //Move Left
            //        mazeData.MoveLeft();
            //        break;
            //    case "right":
            //        //Move Right
            //        mazeData.MoveRight();
            //        break;
            //    case "moveLoc":
            //        //Move Location
            //        //Must read the code then the coordinates
            //        mazeData.MoveLocation(Convert.ToInt32(ReadData()), Convert.ToInt32(ReadData()));
            //        //mazeData.MoveLocation(
            //        break;
            //    case "wallCheck":
            //        //Check Wall
            //        //Must read the code and then the direction
            //        temp = ReadData();
            //        MessageBox.Show(temp);
            //        int tempInt = mazeData.CheckWall(Convert.ToInt32(temp));
            //        MessageBox.Show(tempInt.ToString());
            //        SendData(tempInt.ToString());  //get the maze info and return
            //        //mazeData.CheckWall(
            //        break;
            //    case "curRow":
            //        //Return the current row
            //        SendData(mazeData.CurXLoc.ToString());
            //        break;
            //    case "curCol":
            //        //Return the current column
            //        SendData(mazeData.CurYLoc.ToString());
            //        break;
            //    case "endCheck":
            //        //Return 1 if end or 0 if not
            //        SendData(mazeData.CheckEnd().ToString());
            //        break;
            //    default:
            //        //No a valid code
            //        //Send over message saying so
            //        SendData("Invalid Code: Please try again.");
            //        break;
            //}

           
            //    Invalidate();

            switch (orderNum)
            {
                case "0":
                    break;
                case "up":
                    //Move Up
                    mazeData.MoveUp();
                    break;
                case "down":
                    //Move Down
                    mazeData.MoveDown();
                    break;
                case "left":
                    //Move Left
                    mazeData.MoveLeft();
                    break;
                case "right":
                    //Move Right
                    mazeData.MoveRight();
                    break;
                case "moveLoc":
                    //Move Location
                    //Must read the code then the coordinates
                    string[] item = ReadData().Split(',');
                    //int item = Convert.ToInt32(ReadData());
                    //int item2 = Convert.ToInt32(ReadData());
                    mazeData.MoveLocation(Convert.ToInt32(item[1]), Convert.ToInt32(item[2]));
                    //mazeData.MoveLocation(
                    break;
                case "wallCheckNO":
                    //Check Wall
                    //MessageBox.Show("HEHREHRHEHREHR");
                    //Must read the code and then the direction

                    temp = ReadData();
                    MessageBox.Show(temp);
                    SendData((mazeData.CheckWall(Convert.ToInt32(temp)).ToString()));  //get the maze info and return
                    //mazeData.CheckWall(
                    break;
                case "checkUp":
                    SendData(mazeData.CheckPathUp().ToString());
                    break;
                case "checkDown":
                    SendData(mazeData.CheckPathDown().ToString());
                    break;
                case "checkLeft":
                    SendData(mazeData.CheckPathLeft().ToString());
                    break;
                case "checkRight":
                    SendData(mazeData.CheckPathRight().ToString());
                    break;
                case "curRow":
                    //Return the current row
                    SendData(mazeData.CurXLoc.ToString());
                    break;
                case "curCol":
                    //Return the current column
                    SendData(mazeData.CurYLoc.ToString());
                    break;
                case "endCheck":
                    //Return 1 if end or 0 if not
                    SendData(mazeData.CheckEnd().ToString());
                    break;
                default:
                    //No a valid code
                    //Send over message saying so
                    string[] checkTwice = orderNum.Split(',');
                    if (checkTwice[0] == "moveLoc")
                    {
                        mazeData.MoveLocation(Convert.ToInt32(checkTwice[1]), Convert.ToInt32(checkTwice[2]));
                    }
                    else
                    {

                        SendData("Invalid Code: Please try again.");
                    }
                    break;
            }

            // if (goAhead != -1)
            //{
            Invalidate();
            
        }//End of ManualMoveButtonClick

    }//End of Class
}